@@ -32,7 +32,6 @@
 #include "strlist.h"
 #include "checksums.h"
 #include "names.h"
-#include "chunks.h"
 #include "checkindeb.h"
 #include "reference.h"
 #include "binaries.h"
@@ -42,8 +41,6 @@
 #include "override.h"
 #include "log.h"
 
-extern int verbose;
-
 /* This file includes the code to include binaries, i.e.
    to create the chunk for the Packages.gz-file and
    to put it in the various databases.
@@ -61,7 +58,7 @@ struct debpackage {
 	/* things to be set by deb_read: */
 	struct deb_headers deb;
 	/* things that will still be NULL then: */
-	char *component; //This might be const, too and save some strdups, but...
+	component_t component_atom;
 	/* with deb_calclocations: */
 	const char *filekey;
 	struct strlist filekeys;
@@ -70,7 +67,6 @@ struct debpackage {
 void deb_free(/*@only@*/struct debpackage *pkg) {
 	if( pkg != NULL ) {
 		binaries_debdone(&pkg->deb);
-		free(pkg->component);
 		if( pkg->filekey != NULL )
 			strlist_done(&pkg->filekeys);
 	}
@@ -93,8 +89,6 @@ static retvalue deb_read(/*@out@*/struct debpackage **pkg, const char *filename,
 		r = properversion(deb->deb.sourceversion);
 	if( RET_IS_OK(r) )
 		r = properversion(deb->deb.version);
-	if( !RET_WAS_ERROR(r) )
-		r = properfilenamepart(deb->deb.architecture);
 	if( RET_WAS_ERROR(r) ) {
 		deb_free(deb);
 		return r;
@@ -104,13 +98,13 @@ static retvalue deb_read(/*@out@*/struct debpackage **pkg, const char *filename,
 	return RET_OK;
 }
 
-static retvalue deb_preparelocation(struct debpackage *pkg, const char * const forcecomponent, const char * const forcearchitecture, const char *forcesection, const char *forcepriority, const char * const packagetype, struct distribution *distribution, const struct overrideinfo **oinfo_ptr, const char *debfilename){
-	const struct strlist *components;
+static retvalue deb_preparelocation(struct debpackage *pkg, component_t forcecomponent, architecture_t forcearchitecture, const char *forcesection, const char *forcepriority, packagetype_t packagetype, struct distribution *distribution, const struct overrideinfo **oinfo_ptr, const char *debfilename){
+	const struct atomlist *components;
 	const struct overrideinfo *binoverride;
 	const struct overrideinfo *oinfo;
 	retvalue r;
 
-	if( strcmp(packagetype,"udeb") == 0 ) {
+	if( packagetype == pt_udeb ) {
 		binoverride = distribution->overrides.udeb;
 		components = &distribution->udebcomponents;
 	} else {
@@ -157,38 +151,48 @@ static retvalue deb_preparelocation(struct debpackage *pkg, const char * const f
 
 	r = guess_component(distribution->codename, components,
 			pkg->deb.name, pkg->deb.section,
-			forcecomponent, &pkg->component);
+			forcecomponent, &pkg->component_atom);
 	if( RET_WAS_ERROR(r) )
 		return r;
-	if( verbose > 0 && forcecomponent == NULL ) {
-		fprintf(stderr,"%s: component guessed as '%s'\n",debfilename,pkg->component);
+	if( verbose > 0 && !atom_defined(forcecomponent) ) {
+		fprintf(stderr, "%s: component guessed as '%s'\n", debfilename,
+				atoms_components[pkg->component_atom]);
 	}
 
 	/* some sanity checks: */
 
-	if( forcearchitecture != NULL && strcmp(forcearchitecture,"all") != 0 &&
-			strcmp(pkg->deb.architecture,forcearchitecture) != 0 &&
-			strcmp(pkg->deb.architecture,"all") != 0 ) {
+	if( atom_defined(forcearchitecture) &&
+			forcearchitecture != architecture_all &&
+			pkg->deb.architecture_atom != forcearchitecture &&
+			pkg->deb.architecture_atom != architecture_all ) {
 		fprintf(stderr,"Cannot put '%s' into architecture '%s', as it is '%s'!\n",
-				debfilename,forcearchitecture,pkg->deb.architecture);
+				debfilename,
+				atoms_architectures[forcearchitecture],
+				atoms_architectures[pkg->deb.architecture_atom]);
 		return RET_ERROR;
-	} else if( strcmp(pkg->deb.architecture,"all") != 0 &&
-	    !strlist_in( &distribution->architectures, pkg->deb.architecture )) {
-		(void)fprintf(stderr,"Error looking at '%s': '%s' is not one of the valid architectures: '",
-				debfilename, pkg->deb.architecture);
-		(void)strlist_fprint(stderr, &distribution->architectures);
+	} else if( pkg->deb.architecture_atom != architecture_all &&
+			!atomlist_in(&distribution->architectures,
+				pkg->deb.architecture_atom)) {
+		(void)fprintf(stderr,
+"Error looking at '%s': '%s' is not one of the valid architectures: '",
+				debfilename,
+				atoms_architectures[pkg->deb.architecture_atom]);
+		(void)atomlist_fprint(stderr, at_architecture,
+				&distribution->architectures);
 		(void)fputs("'\n",stderr);
 		return RET_ERROR;
 	}
-	if( !strlist_in(components,pkg->component) ) {
+	if( !atomlist_in(components, pkg->component_atom) ) {
 		fprintf(stderr,
 "Error looking at %s': Would be placed in unavailable component '%s'!\n",
-				debfilename, pkg->component);
+				debfilename,
+				atoms_components[pkg->component_atom]);
 		/* this cannot be ignored as there is not data structure available*/
 		return RET_ERROR;
 	}
 
-	r = binaries_calcfilekeys(pkg->component, &pkg->deb, packagetype, &pkg->filekeys);
+	r = binaries_calcfilekeys(pkg->component_atom, &pkg->deb,
+			packagetype, &pkg->filekeys);
 	if( RET_WAS_ERROR(r) )
 		return r;
 	pkg->filekey = pkg->filekeys.values[0];
@@ -196,7 +200,7 @@ static retvalue deb_preparelocation(struct debpackage *pkg, const char * const f
 }
 
 
-retvalue deb_prepare(/*@out@*/struct debpackage **deb, const char * const forcecomponent, const char * const forcearchitecture, const char *forcesection, const char *forcepriority, const char * const packagetype, struct distribution *distribution, const char *debfilename, const char * const givenfilekey, const struct checksums * checksums, const struct strlist *allowed_binaries, const char *expectedsourcepackage, const char *expectedsourceversion){
+retvalue deb_prepare(/*@out@*/struct debpackage **deb, component_t forcecomponent, architecture_t forcearchitecture, const char *forcesection, const char *forcepriority, packagetype_t packagetype, struct distribution *distribution, const char *debfilename, const char * const givenfilekey, const struct checksums * checksums, const struct strlist *allowed_binaries, const char *expectedsourcepackage, const char *expectedsourceversion){
 	retvalue r;
 	struct debpackage *pkg;
 	const struct overrideinfo *oinfo;
@@ -265,11 +269,10 @@ retvalue deb_prepare(/*@out@*/struct debpackage **deb, const char * const forcec
 	return RET_OK;
 }
 
-retvalue deb_addprepared(const struct debpackage *pkg, struct database *database, const char *forcearchitecture, const char *packagetype, struct distribution *distribution, struct strlist *dereferencedfilekeys, struct trackingdata *trackingdata, bool *usedmarker) {
+retvalue deb_addprepared(const struct debpackage *pkg, struct database *database, architecture_t forcearchitecture, packagetype_t packagetype, struct distribution *distribution, struct trackingdata *trackingdata) {
 	return binaries_adddeb(&pkg->deb, database, forcearchitecture,
-			packagetype, distribution, dereferencedfilekeys,
-			trackingdata,
-			pkg->component, &pkg->filekeys, usedmarker,
+			packagetype, distribution, trackingdata,
+			pkg->component_atom, &pkg->filekeys,
 			pkg->deb.control);
 }
 
@@ -277,14 +280,13 @@ retvalue deb_addprepared(const struct debpackage *pkg, struct database *database
  * putting things with architecture of "all" into <d->architectures> (and also
  * causing error, if it is not one of them otherwise)
  * if component is NULL, guessing it from the section. */
-retvalue deb_add(struct database *database,const char *forcecomponent,const char *forcearchitecture,const char *forcesection,const char *forcepriority,const char *packagetype,struct distribution *distribution,const char *debfilename,int delete,struct strlist *dereferencedfilekeys,/*@null@*/trackingdb tracks){
+retvalue deb_add(struct database *database, component_t forcecomponent, architecture_t forcearchitecture, const char *forcesection, const char *forcepriority, packagetype_t packagetype, struct distribution *distribution, const char *debfilename, int delete, /*@null@*/trackingdb tracks) {
 	struct debpackage *pkg;
 	retvalue r;
 	struct trackingdata trackingdata;
 	const struct overrideinfo *oinfo;
 	char *control;
 	struct checksums *checksums;
-	bool fileused = false;
 
 	causingfile = debfilename;
 
@@ -297,9 +299,7 @@ retvalue deb_add(struct database *database,const char *forcecomponent,const char
 		deb_free(pkg);
 		return r;
 	}
-	r = files_preinclude(database, debfilename, pkg->filekey, &checksums,
-			&fileused);
-	fileused = !fileused;
+	r = files_preinclude(database, debfilename, pkg->filekey, &checksums);
 	if( RET_WAS_ERROR(r) ) {
 		deb_free(pkg);
 		return r;
@@ -309,8 +309,6 @@ retvalue deb_add(struct database *database,const char *forcecomponent,const char
 			pkg->deb.section, pkg->deb.priority, &control);
 	checksums_free(checksums);
 	if( RET_WAS_ERROR(r) ) {
-		if( !fileused )
-			files_deleteandremove(database, pkg->filekey, true, false);
 		deb_free(pkg);
 		return r;
 	}
@@ -322,28 +320,22 @@ retvalue deb_add(struct database *database,const char *forcecomponent,const char
 				pkg->deb.source, pkg->deb.sourceversion,
 				&trackingdata);
 		if( RET_WAS_ERROR(r) ) {
-			if( !fileused )
-				files_deleteandremove(database, pkg->filekey,
-						true, false);
 			deb_free(pkg);
 			return r;
 		}
 	}
 
 	r = binaries_adddeb(&pkg->deb, database, forcearchitecture,
-			packagetype, distribution, dereferencedfilekeys,
+			packagetype, distribution,
 			(tracks!=NULL)?&trackingdata:NULL,
-			pkg->component, &pkg->filekeys, &fileused,
+			pkg->component_atom, &pkg->filekeys,
 			pkg->deb.control);
-	assert( !RET_IS_OK(r) || fileused );
-	if( !fileused )
-		files_deleteandremove(database, pkg->filekey, true, false);
 	RET_UPDATE(distribution->status, r);
 	deb_free(pkg);
 
 	if( tracks != NULL ) {
 		retvalue r2;
-		r2 = trackingdata_finish(tracks, &trackingdata, database, dereferencedfilekeys);
+		r2 = trackingdata_finish(tracks, &trackingdata, database);
 		RET_ENDUPDATE(r,r2);
 	}
 
