@@ -28,7 +28,7 @@
 #include <time.h>
 #include "error.h"
 #include "mprintf.h"
-#include "chunks.h"
+#include "atoms.h"
 #include "sources.h"
 #include "dirs.h"
 #include "names.h"
@@ -41,8 +41,6 @@
 #include "configparser.h"
 #include "distribution.h"
 
-extern int verbose;
-
 static retvalue distribution_free(struct distribution *distribution) {
 	retvalue result,r;
 
@@ -60,18 +58,18 @@ static retvalue distribution_free(struct distribution *distribution) {
 		free(distribution->udeb_override);
 		free(distribution->dsc_override);
 		free(distribution->uploaders);
-		strlist_done(&distribution->udebcomponents);
-		strlist_done(&distribution->architectures);
-		strlist_done(&distribution->components);
+		atomlist_done(&distribution->udebcomponents);
+		atomlist_done(&distribution->architectures);
+		atomlist_done(&distribution->components);
 		strlist_done(&distribution->updates);
 		strlist_done(&distribution->pulls);
 		strlist_done(&distribution->alsoaccept);
 		exportmode_done(&distribution->dsc);
 		exportmode_done(&distribution->deb);
 		exportmode_done(&distribution->udeb);
-		strlist_done(&distribution->contents_architectures);
-		strlist_done(&distribution->contents_components);
-		strlist_done(&distribution->contents_ucomponents);
+		atomlist_done(&distribution->contents_architectures);
+		atomlist_done(&distribution->contents_components);
+		atomlist_done(&distribution->contents_ucomponents);
 		override_free(distribution->overrides.deb);
 		override_free(distribution->overrides.udeb);
 		override_free(distribution->overrides.dsc);
@@ -108,22 +106,47 @@ void distribution_unloadoverrides(struct distribution *distribution) {
 static retvalue createtargets(struct distribution *distribution) {
 	retvalue r;
 	int i,j;
-	const char *arch,*comp;
 	struct target *t;
 	struct target *last = NULL;
+	bool has_source = false;
 
 	for( i = 0 ; i < distribution->components.count ; i++ ) {
-		comp = distribution->components.values[i];
+		component_t c = distribution->components.atoms[i];
 		for( j = 0 ; j < distribution->architectures.count ; j++ ) {
-			arch = distribution->architectures.values[j];
-			if( strcmp(arch,"source") != 0 ) {
-				if( strcmp(arch,"all") == 0 && verbose >= 0 ) {
-					fprintf(stderr,
-"WARNING: Distribution %s contains an architecture called 'all'.\n",
-						distribution->codename);
-				}
+			architecture_t a = distribution->architectures.atoms[j];
 
-				r = target_initialize_binary(distribution->codename,comp,arch,&distribution->deb,&t);
+			if( a == architecture_source ) {
+				has_source = true;
+				continue;
+			}
+			if( a == architecture_all ) {
+				fprintf(stderr,
+"Error: Distribution %s contains an architecture called 'all'.\n",
+						distribution->codename);
+				return RET_ERROR;
+			}
+
+			r = target_initialize_binary(
+					distribution->codename,
+					c, a,
+					&distribution->deb,
+					distribution->readonly, &t);
+			if( RET_IS_OK(r) ) {
+				if( last != NULL ) {
+					last->next = t;
+				} else {
+					distribution->targets = t;
+				}
+				last = t;
+			}
+			if( RET_WAS_ERROR(r) )
+				return r;
+			if( atomlist_in(&distribution->udebcomponents, c) ) {
+				r = target_initialize_ubinary(
+						distribution->codename,
+						c, a,
+						&distribution->udeb,
+						distribution->readonly, &t);
 				if( RET_IS_OK(r) ) {
 					if( last != NULL ) {
 						last->next = t;
@@ -134,28 +157,16 @@ static retvalue createtargets(struct distribution *distribution) {
 				}
 				if( RET_WAS_ERROR(r) )
 					return r;
-				if( strlist_in(&distribution->udebcomponents,comp) ) {
-					r = target_initialize_ubinary(distribution->codename,comp,arch,&distribution->udeb,&t);
-					if( RET_IS_OK(r) ) {
-						if( last != NULL ) {
-							last->next = t;
-						} else {
-							distribution->targets = t;
-						}
-						last = t;
-					}
-					if( RET_WAS_ERROR(r) )
-						return r;
 
-				}
 			}
-
 		}
 		/* check if this distribution contains source
 		 * (yes, yes, source is not really an architecture, but
 		 *  the .changes files started with this...) */
-		if( strlist_in(&distribution->architectures,"source") ) {
-			r = target_initialize_source(distribution->codename,comp,&distribution->dsc,&t);
+		if( has_source ) {
+			r = target_initialize_source(distribution->codename,
+					c, &distribution->dsc,
+					distribution->readonly, &t);
 			if( last != NULL ) {
 				last->next = t;
 			} else {
@@ -201,19 +212,20 @@ CFstartparse(distribution) {
 	return RET_OK;
 }
 
-static bool notpropersuperset(const struct strlist *allowed, const char *allowedname,
-		const struct strlist *check, const char *checkname,
+static bool notpropersuperset(const struct atomlist *allowed, const char *allowedname,
+		const struct atomlist *check, const char *checkname,
+		const char **atoms,
 		const struct configiterator *iter, const struct distribution *d) {
-	const char *missing;
+	atom_t missing;
 
-	if( !strlist_subset(allowed, check, &missing) ) {
+	if( !atomlist_subset(allowed, check, &missing) ) {
 		fprintf(stderr,
 "In distribution description of '%s' (line %u to %u in %s):\n"
 "%s contains '%s' not found in %s!\n",
 				d->codename,
 				d->firstline, d->lastline,
 				config_filename(iter),
-				checkname, missing, allowedname);
+				checkname, atoms[missing], allowedname);
 		return true;
 	}
 	return false;
@@ -247,17 +259,21 @@ CFfinishparse(distribution) {
 
 	if( notpropersuperset(&n->architectures, "Architectures",
 			    &n->contents_architectures, "ContentsArchitectures",
+			    atoms_architectures,
 			    iter, n) ||
 	    notpropersuperset(&n->components, "Components",
 			    &n->contents_components, "ContentsComponents",
+			    atoms_components,
 			    iter, n) ||
 	    notpropersuperset(&n->udebcomponents, "UDebComponents",
 			    &n->contents_ucomponents, "ContentsUComponents",
+			    atoms_components,
 			    iter, n) ||
 	    // TODO: instead of checking here make sure it can have more
 	    // in the rest of the code...:
 	    notpropersuperset(&n->components, "Components",
 			    &n->udebcomponents, "UDebComponents",
+			    atoms_components,
 			    iter, n) ) {
 		(void)distribution_free(n);
 		return RET_ERROR;
@@ -309,6 +325,7 @@ CFallSETPROC(distribution, suite)
 CFallSETPROC(distribution, version)
 CFallSETPROC(distribution, origin)
 CFallSETPROC(distribution, notautomatic)
+CFtruthSETPROC2(distribution, readonly, readonly)
 CFallSETPROC(distribution, label)
 CFallSETPROC(distribution, description)
 CFkeySETPROC(distribution, signwith)
@@ -316,20 +333,21 @@ CFfileSETPROC(distribution, deb_override)
 CFfileSETPROC(distribution, udeb_override)
 CFfileSETPROC(distribution, dsc_override)
 CFfileSETPROC(distribution, uploaders)
-CFuniqstrlistSETPROC(distribution, udebcomponents)
 CFuniqstrlistSETPROC(distribution, alsoaccept)
 CFstrlistSETPROC(distribution, updates)
 CFstrlistSETPROC(distribution, pulls)
-CFuniqstrlistSETPROCset(distribution, contents_architectures)
-CFuniqstrlistSETPROCset(distribution, contents_components)
-CFuniqstrlistSETPROCset(distribution, contents_ucomponents)
+CFinternatomsSETPROC(distribution, components, checkforcomponent, at_component)
+CFinternatomsSETPROC(distribution, architectures, checkforarchitecture, at_architecture)
+CFatomsublistSETPROC(distribution, contents_architectures, at_architecture, architectures, "Architectures")
+CFatomsublistSETPROC(distribution, contents_components, at_component, components, "Components")
+CFatomsublistSETPROC(distribution, udebcomponents, at_component, components, "Components")
+CFatomsublistSETPROC(distribution, contents_ucomponents, at_component, udebcomponents, "UDebComponents")
 CFexportmodeSETPROC(distribution, udeb)
 CFexportmodeSETPROC(distribution, deb)
 CFexportmodeSETPROC(distribution, dsc)
-CFcheckuniqstrlistSETPROC(distribution, components, checkforcomponent)
-CFcheckuniqstrlistSETPROC(distribution, architectures, checkforarchitecture)
 CFcheckvalueSETPROC(distribution, codename, checkforcodename)
 CFcheckvalueSETPROC(distribution, fakecomponentprefix, checkfordirectoryandidentifier)
+CFtimespanSETPROC(distribution, validfor)
 
 CFUSETPROC(distribution, Contents) {
 	CFSETPROCVAR(distribution, d);
@@ -364,6 +382,7 @@ static const struct configfield distributionconfigfields[] = {
 	CF("NotAutomatic",	distribution,	notautomatic),
 	CF("Origin",		distribution,	origin),
 	CF("Pull",		distribution,	pulls),
+	CF("ReadOnly",		distribution,	readonly),
 	CF("SignWith",		distribution,	signwith),
 	CF("Suite",		distribution,	suite),
 	CF("Tracking",		distribution,	Tracking),
@@ -372,6 +391,7 @@ static const struct configfield distributionconfigfields[] = {
 	CF("UDebOverride",	distribution,	udeb_override),
 	CF("Update",		distribution,	updates),
 	CF("Uploaders",		distribution,	uploaders),
+	CF("ValidFor",		distribution,	validfor),
 	CF("Version",		distribution,	version)
 };
 
@@ -417,58 +437,8 @@ retvalue distribution_readall(struct distribution **distributions) {
 	return RET_OK;
 }
 
-/* call <action> for each part of <distribution>, within initpackagesdb/closepackagesdb */
-retvalue distribution_foreach_rwopenedpart(struct distribution *distribution,struct database *database,const char *component,const char *architecture,const char *packagetype,distribution_each_action action,void *data) {
-	retvalue result,r;
-	struct target *t;
-
-	result = RET_NOTHING;
-	for( t = distribution->targets ; t != NULL ; t = t->next ) {
-		if( !target_matches(t, component, architecture, packagetype) )
-			continue;
-		r = target_initpackagesdb(t, database, READWRITE);
-		RET_UPDATE(result, r);
-		if( RET_WAS_ERROR(r) )
-			return result;
-		r = action(data, t, distribution);
-		RET_UPDATE(result, r);
-		// TODO: how to seperate this in those affecting distribution
-		// and those that do not?
-		RET_UPDATE(distribution->status, r);
-		r = target_closepackagesdb(t);
-		RET_UPDATE(distribution->status, r);
-		RET_UPDATE(result, r);
-		if( RET_WAS_ERROR(result) )
-			return result;
-	}
-	return result;
-}
-
-/* call <action> for each part of <distribution>, within initpackagesdb/closepackagesdb */
-retvalue distribution_foreach_roopenedpart(struct distribution *distribution,struct database *database,const char *component,const char *architecture,const char *packagetype,distribution_each_action action,void *data) {
-	retvalue result,r;
-	struct target *t;
-
-	result = RET_NOTHING;
-	for( t = distribution->targets ; t != NULL ; t = t->next ) {
-		if( !target_matches(t, component, architecture, packagetype) )
-			continue;
-		r = target_initpackagesdb(t, database, READONLY);
-		RET_UPDATE(result, r);
-		if( RET_WAS_ERROR(r) )
-			return result;
-		r = action(data, t, distribution);
-		RET_UPDATE(result, r);
-		r = target_closepackagesdb(t);
-		RET_UPDATE(result, r);
-		if( RET_WAS_ERROR(result) )
-			return result;
-	}
-	return result;
-}
-
 /* call <action> for each package */
-retvalue distribution_foreach_package(struct distribution *distribution, struct database *database, const char *component, const char *architecture, const char *packagetype, each_package_action action, each_target_action target_action, void *data) {
+retvalue distribution_foreach_package(struct distribution *distribution, struct database *database, component_t component, architecture_t architecture, packagetype_t packagetype, each_package_action action, each_target_action target_action, void *data) {
 	retvalue result,r;
 	struct target *t;
 	struct target_cursor iterator IFSTUPIDCC(=TARGET_CURSOR_ZERO);
@@ -504,19 +474,19 @@ retvalue distribution_foreach_package(struct distribution *distribution, struct
 	return result;
 }
 
-retvalue distribution_foreach_package_c(struct distribution *distribution, struct database *database, const struct strlist *components, const char *architecture, const char *packagetype, each_package_action action, void *data) {
+retvalue distribution_foreach_package_c(struct distribution *distribution, struct database *database, const struct atomlist *components, architecture_t architecture, packagetype_t packagetype, each_package_action action, void *data) {
 	retvalue result,r;
 	struct target *t;
 	const char *package, *control;
 	struct target_cursor iterator IFSTUPIDCC(=TARGET_CURSOR_ZERO);
 
+	assert( components != NULL );
+
 	result = RET_NOTHING;
 	for( t = distribution->targets ; t != NULL ; t = t->next ) {
-		if( components != NULL && !strlist_in(components, t->component) )
+		if( !atomlist_in(components, t->component_atom) )
 			continue;
-		if( architecture != NULL && strcmp(architecture,t->architecture) != 0 )
-			continue;
-		if( packagetype != NULL && strcmp(packagetype,t->packagetype) != 0 )
+		if( !target_matches(t, atom_unknown, architecture, packagetype) )
 			continue;
 		r = target_openiterator(t, database, READONLY, &iterator);
 		RET_UPDATE(result, r);
@@ -537,26 +507,42 @@ retvalue distribution_foreach_package_c(struct distribution *distribution, struc
 	return result;
 }
 
-struct target *distribution_gettarget(const struct distribution *distribution,const char *component,const char *architecture,const char *packagetype) {
+struct target *distribution_gettarget(const struct distribution *distribution, component_t component, architecture_t architecture, packagetype_t packagetype) {
 	struct target *t = distribution->targets;
 
+	assert( atom_defined(component) );
+	assert( atom_defined(architecture) );
+	assert( atom_defined(packagetype) );
+
 	// TODO: think about making read only access and only alowing readwrite when lookedat is set
 
-	while( t != NULL && ( strcmp(t->component,component) != 0 || strcmp(t->architecture,architecture) != 0 || strcmp(t->packagetype,packagetype) != 0 )) {
+	while( t != NULL &&
+			( t->component_atom != component ||
+			  t->architecture_atom != architecture ||
+			  t->packagetype_atom != packagetype )) {
 		t = t->next;
 	}
 	return t;
 }
 
-struct target *distribution_getpart(const struct distribution *distribution,const char *component,const char *architecture,const char *packagetype) {
+struct target *distribution_getpart(const struct distribution *distribution, component_t component, architecture_t architecture, packagetype_t packagetype) {
 	struct target *t = distribution->targets;
 
-	while( t != NULL && ( strcmp(t->component,component) != 0 || strcmp(t->architecture,architecture) != 0 || strcmp(t->packagetype,packagetype) != 0 )) {
+	assert( atom_defined(component) );
+	assert( atom_defined(architecture) );
+	assert( atom_defined(packagetype) );
+
+	while( t != NULL &&
+			( t->component_atom != component ||
+			  t->architecture_atom != architecture ||
+			  t->packagetype_atom != packagetype )) {
 		t = t->next;
 	}
 	if( t == NULL ) {
 		fprintf(stderr, "Internal error in distribution_getpart: Bogus request for c='%s' a='%s' t='%s' in '%s'!\n",
-				component, architecture, packagetype,
+				atoms_components[component],
+				atoms_architectures[architecture],
+				atoms_packagetypes[packagetype],
 				distribution->codename);
 		abort();
 	}
@@ -564,7 +550,7 @@ struct target *distribution_getpart(const struct distribution *distribution,cons
 }
 
 /* mark all distributions matching one of the first argc argv */
-retvalue distribution_match(struct distribution *alldistributions, int argc, const char *argv[], bool lookedat) {
+retvalue distribution_match(struct distribution *alldistributions, int argc, const char *argv[], bool lookedat, bool allowreadonly) {
 	struct distribution *d;
 	bool found[argc], unusable_as_suite[argc];
 	struct distribution *has_suite[argc];
@@ -574,6 +560,8 @@ retvalue distribution_match(struct distribution *alldistributions, int argc, con
 
 	if( argc <= 0 ) {
 		for( d = alldistributions ; d != NULL ; d = d->next ) {
+			if( !allowreadonly && d->readonly )
+				continue;
 			d->selected = true;
 			d->lookedat = lookedat;
 		}
@@ -591,6 +579,12 @@ retvalue distribution_match(struct distribution *alldistributions, int argc, con
 				d->selected = true;
 				if( lookedat )
 					d->lookedat = lookedat;
+				if( !allowreadonly && d->readonly ) {
+					fprintf(stderr,
+"Error: %s is readonly, so operation not allowed!\n",
+							d->codename);
+					return RET_ERROR;
+				}
 			} else if( d->suite != NULL &&
 					strcmp(argv[i], d->suite) == 0 ) {
 				if( has_suite[i] != NULL )
@@ -602,12 +596,20 @@ retvalue distribution_match(struct distribution *alldistributions, int argc, con
 	for( i = 0 ; i < argc ; i++ ) {
 		if( !found[i] ) {
 			if( has_suite[i] != NULL && !unusable_as_suite[i] ) {
+				if( !allowreadonly && has_suite[i]->readonly ) {
+					fprintf(stderr,
+"Error: %s is readonly, so operation not allowed!\n",
+							has_suite[i]->codename);
+					return RET_ERROR;
+				}
 				has_suite[i]->selected = true;
 				if( lookedat )
 					has_suite[i]->lookedat = lookedat;
 				continue;
 			}
-			fprintf(stderr, "No distribution definition of '%s' found in distributions'!\n", argv[i]);
+			fprintf(stderr,
+"No distribution definition of '%s' found in '%s/distributions'!\n",
+				argv[i], global.confdir);
 			if( unusable_as_suite[i] )
 				fprintf(stderr,
 "(It is not the codename of any distribution and there are multiple\n"
@@ -694,7 +696,7 @@ retvalue distribution_snapshot(struct distribution *distribution, struct databas
 	if( id == NULL )
 		return RET_ERROR_OOM;
 	r = distribution_foreach_package(distribution, database,
-			NULL, NULL, NULL,
+			atom_unknown, atom_unknown, atom_unknown,
 			package_referenceforsnapshot, NULL, id);
 	free(id);
 	RET_UPDATE(result,r);
@@ -708,6 +710,12 @@ static retvalue export(struct distribution *distribution, struct database *datab
 
 	assert( distribution != NULL );
 
+	if( distribution->readonly ) {
+		fprintf(stderr, "Error: trying to re-export read-only distribution %s\n",
+				distribution->codename);
+		return RET_ERROR;
+	}
+
 	r = release_init(&release, database,
 			distribution->codename, distribution->suite,
 			distribution->fakecomponentprefix);
@@ -792,7 +800,7 @@ retvalue distribution_exportlist(enum exportwhen when, struct distribution *dist
 	if( when == EXPORT_NEVER ) {
 		if( verbose > 10 )
 			fprintf(stderr, "Not exporting anything as --export=never specified\n");
-		return distribution_freelist(distributions);
+		return RET_NOTHING;
 	}
 
 	for( d=distributions; d != NULL; d = d->next ) {
@@ -992,6 +1000,12 @@ void distribution_unloaduploaders(struct distribution *distribution) {
 retvalue distribution_prepareforwriting(struct distribution *distribution) {
 	retvalue r;
 
+	if( distribution->readonly ) {
+		fprintf(stderr, "Error: distribution %s is read-only. Current operation not possible because it needs write access.\n",
+				distribution->codename);
+		return RET_ERROR;
+	}
+
 	if( distribution->logger != NULL ) {
 		r = logger_prepare(distribution->logger);
 		if( RET_WAS_ERROR(r) )
@@ -1002,19 +1016,21 @@ retvalue distribution_prepareforwriting(struct distribution *distribution) {
 }
 
 /* delete every package decider returns RET_OK for */
-retvalue distribution_remove_packages(struct distribution *distribution, struct database *database, const char *component, const char *architecture, const char *packagetype, each_package_action decider, struct strlist *dereferenced, struct trackingdata *trackingdata, void *data) {
+retvalue distribution_remove_packages(struct distribution *distribution, struct database *database, component_t component, architecture_t architecture, packagetype_t packagetype, each_package_action decider, struct trackingdata *trackingdata, void *data) {
 	retvalue result,r;
 	struct target *t;
 	struct target_cursor iterator;
 	const char *package, *control;
 
+	if( distribution->readonly ) {
+		fprintf(stderr, "Error: trying to delete packages in read-only distribution %s.\n",
+				distribution->codename);
+		return RET_ERROR;
+	}
+
 	result = RET_NOTHING;
 	for( t = distribution->targets ; t != NULL ; t = t->next ) {
-		if( component != NULL && strcmp(component,t->component) != 0 )
-			continue;
-		if( architecture != NULL && strcmp(architecture,t->architecture) != 0 )
-			continue;
-		if( packagetype != NULL && strcmp(packagetype,t->packagetype) != 0 )
+		if( !target_matches(t, component, architecture, packagetype) )
 			continue;
 		r = target_openiterator(t, database, READWRITE, &iterator);
 		RET_UPDATE(result, r);
@@ -1029,7 +1045,7 @@ retvalue distribution_remove_packages(struct distribution *distribution, struct
 			if( RET_IS_OK(r) ) {
 				r = target_removepackage_by_cursor(&iterator,
 					distribution->logger, database,
-					dereferenced, trackingdata);
+					trackingdata);
 				RET_UPDATE(result, r);
 				RET_UPDATE(distribution->status, r);
 			}
