@@ -9,12 +9,15 @@
 #include "strlist.h"
 #warning "What's hapening here?"
 #endif
-#ifndef REPREPRO_FILES_H
-#include "files.h"
+#ifndef REPREPRO_DATABASE_H
+#include "database.h"
 #endif
 #ifndef REPREPRO_APTMETHOD_H
 #include "aptmethod.h"
 #endif
+#ifndef REPREPRO_CHECKSUMS_H
+#include "checksums.h"
+#endif
 #ifndef REPREPRO_FREESPACE_H
 #include "freespace.h"
 #endif
@@ -27,7 +30,7 @@ struct downloadcache {
 };
 
 /* Initialize a new download session */
-retvalue downloadcache_initialize(const char *dbdir,enum spacecheckmode mode,off_t reserveddb,off_t reservedother,struct downloadcache **download);
+retvalue downloadcache_initialize(struct database *,enum spacecheckmode mode,off_t reserveddb,off_t reservedother,struct downloadcache **download);
 
 /* free all memory */
 retvalue downloadcache_free(/*@null@*//*@only@*/struct downloadcache *download);
@@ -35,12 +38,11 @@ retvalue downloadcache_free(/*@null@*//*@only@*/struct downloadcache *download);
 /* queue a new file to be downloaded:
  * results in RET_ERROR_WRONG_MD5, if someone else already asked
  * for the same destination with other md5sum created. */
-retvalue downloadcache_add(struct downloadcache *cache,filesdb filesdb,struct aptmethod *method,const char *orig,const char *filekey,const char *md5sum);
+retvalue downloadcache_add(struct downloadcache *, struct database *, struct aptmethod *, const char *orig, const char *filekey, const struct checksums *);
 
 /* some as above, only for more files... */
-retvalue downloadcache_addfiles(struct downloadcache *cache,filesdb filesdb,
+retvalue downloadcache_addfiles(struct downloadcache *cache,struct database *,
 		struct aptmethod *method,
-		const struct strlist *origfiles,
-		const struct strlist *filekeys,
-		const struct strlist *md5sums);
+		const struct checksumsarray *origfiles,
+		const struct strlist *filekeys);
 #endif
