@@ -5,7 +5,8 @@
 void closefrom(int);
 #endif
 void markcloseonexec(int);
-void deletefile(const char *);
+int deletefile(const char *);
 bool isregularfile(const char *);
+bool isdirectory(const char *fullfilename);
 
 #endif
