@@ -1,5 +1,5 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2003,2004,2005,2007 Bernhard R. Link
+ *  Copyright (C) 2003,2004,2005,2007,2009 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
@@ -86,11 +86,13 @@ void release_free(struct release *release) {
 		free(e->relativefilename);
 		checksums_free(e->checksums);
 		free(e->fullfinalfilename);
+		if( !global.keeptemporaries && e->fulltemporaryfilename != NULL)
+			unlink(e->fulltemporaryfilename);
 		free(e->fulltemporaryfilename);
 		free(e);
 	}
 	if( release->signedfile != NULL )
-		signedfile_free(release->signedfile);
+		signedfile_free(release->signedfile, !global.keeptemporaries);
 	if( release->cachedb != NULL ) {
 		table_close(release->cachedb);
 	}
@@ -248,6 +250,28 @@ retvalue release_addnew(struct release *release,/*@only@*/char *reltmpfile,/*@on
 			checksums, finalfilename, filename);
 }
 
+retvalue release_addsilentnew(struct release *release,/*@only@*/char *reltmpfile,/*@only@*/char *relfilename) {
+	char *filename, *finalfilename;
+
+	filename = calc_dirconcat(release->dirofdist,reltmpfile);
+	if( filename == NULL ) {
+		free(reltmpfile);
+		free(relfilename);
+		return RET_ERROR_OOM;
+	}
+	free(reltmpfile);
+	finalfilename = calc_dirconcat(release->dirofdist,relfilename);
+	if( finalfilename == NULL ) {
+		free(relfilename);
+		free(filename);
+		return RET_ERROR_OOM;
+	}
+	free(relfilename);
+	release->new = true;
+	return newreleaseentry(release, NULL,
+			NULL, finalfilename, filename);
+}
+
 retvalue release_addold(struct release *release,/*@only@*/char *relfilename) {
 	retvalue r;
 	char *filename;
@@ -1281,9 +1305,10 @@ retvalue release_prepare(struct release *release, struct distribution *distribut
 			writechar('\n');
 		}
 	}
-	r = signedfile_prepare(release->signedfile, distribution->signwith);
+	r = signedfile_prepare(release->signedfile, distribution->signwith,
+			!global.keeptemporaries);
 	if( RET_WAS_ERROR(r) ) {
-		signedfile_free(release->signedfile);
+		signedfile_free(release->signedfile, !global.keeptemporaries);
 		release->signedfile = NULL;
 		return r;
 	}
@@ -1301,7 +1326,8 @@ retvalue release_finish(/*@only@*/struct release *release, struct distribution *
 	result = RET_OK;
 
 	for( file = release->files ; file != NULL ; file = file->next ) {
-		if( file->relativefilename == NULL ) {
+		if( file->relativefilename == NULL
+				&& file->fullfinalfilename == NULL ) {
 			assert(file->fulltemporaryfilename != NULL );
 			e = unlink(file->fulltemporaryfilename);
 			if( e < 0 ) {
@@ -1311,6 +1337,8 @@ retvalue release_finish(/*@only@*/struct release *release, struct distribution *
 					e, file->fulltemporaryfilename,
 					strerror(e));
 			}
+			free(file->fulltemporaryfilename);
+			file->fulltemporaryfilename = NULL;
 		} else if( file->fulltemporaryfilename != NULL ) {
 			e = rename(file->fulltemporaryfilename,
 					file->fullfinalfilename);
@@ -1328,8 +1356,11 @@ retvalue release_finish(/*@only@*/struct release *release, struct distribution *
 					return r;
 				}
 				RET_UPDATE(result,r);
-			} else
+			} else {
 				somethingwasdone = true;
+				free(file->fulltemporaryfilename);
+				file->fulltemporaryfilename = NULL;
+			}
 		}
 	}
 	r = signedfile_finalize(release->signedfile, &somethingwasdone);
