@@ -239,7 +239,8 @@ retvalue target_removereadpackage(struct target *target, struct logger *logger,
 			logger_log(logger, target, name,
 					NULL, oldpversion,
 					NULL, oldcontrol,
-					NULL, &files);
+					NULL, &files,
+					NULL, NULL);
 		r = references_delete(database, target->identifier, &files,
 				NULL);
 		RET_UPDATE(result, r);
@@ -321,7 +322,8 @@ retvalue target_removepackage_by_cursor(struct target_cursor *tc, struct logger
 			logger_log(logger, target, name,
 					NULL, oldpversion,
 					NULL, control,
-					NULL, &files);
+					NULL, &files,
+					NULL, NULL);
 		r = references_delete(database, target->identifier, &files,
 				NULL);
 		RET_UPDATE(result, r);
@@ -340,7 +342,8 @@ static retvalue addpackages(struct target *target, struct database *database,
 		/*@null@*/struct logger *logger,
 		/*@null@*/struct trackingdata *trackingdata,
 		architecture_t architecture,
-		/*@null@*//*@only@*/char *oldsource,/*@null@*//*@only@*/char *oldsversion) {
+		/*@null@*//*@only@*/char *oldsource,/*@null@*//*@only@*/char *oldsversion,
+		/*@null@*/const char *causingrule, /*@null@*/const char *suitefrom) {
 
 	retvalue result,r;
 	struct table *table = target->packages;
@@ -385,7 +388,7 @@ static retvalue addpackages(struct target *target, struct database *database,
 		logger_log(logger, target, packagename,
 				version, oldversion,
 				controlchunk, oldcontrolchunk,
-				files, oldfiles);
+				files, oldfiles, causingrule, suitefrom);
 
 	r = trackingdata_insert(trackingdata, filetype, files,
 			oldsource, oldsversion, oldfiles, database);
@@ -403,7 +406,7 @@ static retvalue addpackages(struct target *target, struct database *database,
 	return result;
 }
 
-retvalue target_addpackage(struct target *target, struct logger *logger, struct database *database, const char *name, const char *version, const char *control, const struct strlist *filekeys, bool downgrade, struct trackingdata *trackingdata, enum filetype filetype) {
+retvalue target_addpackage(struct target *target, struct logger *logger, struct database *database, const char *name, const char *version, const char *control, const struct strlist *filekeys, bool downgrade, struct trackingdata *trackingdata, enum filetype filetype, const char *causingrule, const char *suitefrom) {
 	struct strlist oldfilekeys,*ofk;
 	char *oldcontrol,*oldsource,*oldsversion;
 	char *oldpversion;
@@ -487,7 +490,8 @@ retvalue target_addpackage(struct target *target, struct logger *logger, struct
 			version, oldpversion,
 			filekeys, ofk,
 			logger,
-			trackingdata, filetype, oldsource, oldsversion);
+			trackingdata, filetype, oldsource, oldsversion,
+			causingrule, suitefrom);
 	if( RET_IS_OK(r) ) {
 		target->wasmodified = true;
 	}
