@@ -4,6 +4,9 @@ set -e
 
 source $(dirname $0)/test.inc
 
+runtest exporthooks
+runtest updatecorners
+runtest packagediff
 runtest includeextra
 runtest atoms
 runtest trackingcorruption
